<?php
/**
 * RuleForm.php
 * Copyright (c) 2019 james@firefly-iii.org
 *
 * This file is part of Firefly III (https://github.com/firefly-iii).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

declare(strict_types=1);

namespace FireflyIII\Support\Form;

use FireflyIII\Models\RuleGroup;
use FireflyIII\Repositories\RuleGroup\RuleGroupRepositoryInterface;
use Form;
use Illuminate\Support\HtmlString;

/**
 * Class RuleForm
 * See reference nr. 31
 */
class RuleForm
{
    use FormSupport;

    /**
     * @param string     $name
     * @param mixed      $value
     * @param array|null $options
     *
     * @return string
     */
    public function ruleGroupList(string $name, $value = null, array $options = null): string
    {
        /** @var RuleGroupRepositoryInterface $groupRepos */
        $groupRepos = app(RuleGroupRepositoryInterface::class);

        // get all currencies:
        $list  = $groupRepos->get();
        $array = [];
        /** @var RuleGroup $group */
        foreach ($list as $group) {
            $array[$group->id] = $group->title;
        }

        return $this->select($name, $array, $value, $options);
    }

    /**
     * @param string     $name
     * @param null       $value
     * @param array|null $options
     *
     * @return HtmlString
     */
    public function ruleGroupListWithEmpty(string $name, $value = null, array $options = null): HtmlString
    {
        $options          = $options ?? [];
        $options['class'] = 'form-control';
        /** @var RuleGroupRepositoryInterface $groupRepos */
        $groupRepos = app(RuleGroupRepositoryInterface::class);

        // get all currencies:
        $list  = $groupRepos->get();
        $array = [
            0 => (string) trans('firefly.none_in_select_list'),
        ];
        /** @var RuleGroup $group */
        foreach ($list as $group) {
            if (array_key_exists('hidden', $options) && (int) $options['hidden'] !== $group->id) {
                $array[$group->id] = $group->title;
            }
        }

        return Form::select($name, $array, $value, $options);
    }
}
